@@ -50,7 +50,6 @@ enum
   PROP_PASSWORD,
   PROP_NAME,
   PROP_BEST_NAME,
-  PROP_ICON,
   PROP_FULLSCREEN
 };
 
@@ -162,10 +161,6 @@ vinagre_connection_get_property (GObject *object, guint prop_id, GValue *value,
 	g_value_set_string (value, conn->priv->password);
 	break;
 
-      case PROP_ICON:
-	g_value_set_object (value, vinagre_connection_get_icon (conn));
-	break;
-
       case PROP_FULLSCREEN:
 	g_value_set_boolean (value, conn->priv->fullscreen);
 	break;
@@ -187,6 +182,8 @@ vinagre_connection_get_property (GObject *object, guint prop_id, GValue *value,
 static void
 default_fill_writer (VinagreConnection *conn, xmlTextWriter *writer)
 {
+  if (conn->priv->protocol)
+    xmlTextWriterWriteElement (writer, "protocol", conn->priv->protocol);
   xmlTextWriterWriteElement (writer, "name", conn->priv->name);
   xmlTextWriterWriteElement (writer, "host", conn->priv->host);
   xmlTextWriterWriteFormatElement (writer, "port", "%d", conn->priv->port);
@@ -289,7 +286,7 @@ vinagre_connection_class_init (VinagreConnectionClass *klass)
                                                       G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (object_class,
-                                   PROP_NAME,
+                                   PROP_USERNAME,
                                    g_param_spec_string ("username",
                                                         "username",
 	                                                "username (if any) necessary for complete this connection",
@@ -335,16 +332,6 @@ vinagre_connection_class_init (VinagreConnectionClass *klass)
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_BLURB));
 
-  g_object_class_install_property (object_class,
-                                   PROP_ICON,
-                                   g_param_spec_object ("icon",
-                                                        "icon",
-	                                                "icon of this connection",
-                                                        GDK_TYPE_PIXBUF,
-	                                                G_PARAM_READABLE |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_BLURB));
   g_object_class_install_property (object_class,
                                    PROP_FULLSCREEN,
                                    g_param_spec_boolean ("fullscreen",
@@ -383,7 +370,10 @@ vinagre_connection_set_host (VinagreConnection *conn,
   g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
 
   g_free (conn->priv->host);
-  conn->priv->host = g_strdup (host);
+  if (host)
+    conn->priv->host = g_strdup (g_strstrip ((gchar *)host));
+  else
+    conn->priv->host = NULL;
 }
 const gchar *
 vinagre_connection_get_host (VinagreConnection *conn)
@@ -476,29 +466,22 @@ vinagre_connection_get_name (VinagreConnection *conn)
   return conn->priv->name;
 }
 
-GdkPixbuf *
-vinagre_connection_get_icon (VinagreConnection *conn)
-{
-  GdkPixbuf         *pixbuf;
-  GtkIconTheme      *icon_theme;
-  gchar             *icon_name;
-
-  g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), NULL);
-
-  icon_name = g_strdup_printf ("application-x-%s", conn->priv->protocol);
-  icon_theme = gtk_icon_theme_get_default ();
-  pixbuf = gtk_icon_theme_load_icon (icon_theme,
-				     icon_name,
-				     16,
-				     0,
-				     NULL);
-
-  g_free (icon_name);
-  return pixbuf;
-}
-
+/**
+ * vinagre_connection_split_string:
+ * @uri: The URI to be splitted.
+ * @known_protocol: The protocol, if it's known. NULL otherwise.
+ * @protocol: Will hold the protocol of this URI
+ * @host: Will hold the host of this URI
+ * @port: Will hold the port of this URI
+ * @error_msg: Will hold an error message in case of fail
+ *
+ * Splits a URI into its several parts.
+ *
+ * Returns: %TRUE if the URI is splitted successfuly. FALSE otherwise.
+ */
 gboolean
 vinagre_connection_split_string (const gchar *uri,
+				 const gchar *known_protocol,
 				 gchar       **protocol,
 				 gchar       **host,
 				 gint         *port,
@@ -517,12 +500,18 @@ vinagre_connection_split_string (const gchar *uri,
   url = g_strsplit (uri, "://", 2);
   if (g_strv_length (url) == 2)
     {
-      *protocol = g_strdup (url[0]);
+      if (known_protocol)
+	*protocol = g_strdup (known_protocol);
+      else
+	*protocol = g_strdup (url[0]);
       lhost = url[1];
     }
   else
     {
-      *protocol = g_strdup ("vnc");
+      if (known_protocol)
+	*protocol = g_strdup (known_protocol);
+      else
+	*protocol = g_strdup ("vnc");
       lhost = (gchar *) uri;
     }
 
@@ -563,7 +552,7 @@ vinagre_connection_split_string (const gchar *uri,
         lport += 5900;
     }
 
-  lhost = ipv6_host[0] ? ipv6_host : server[0];
+  lhost = ipv6_host[0] ? ipv6_host : (server[0][0] ? server[0] : "localhost");
 
   *host = g_strdup (lhost);
   *port = lport;
@@ -582,7 +571,7 @@ vinagre_connection_new_from_string (const gchar *uri, gchar **error_msg, gboolea
   gchar  *host, *protocol;
   VinagrePlugin *plugin;
 
-  if (!vinagre_connection_split_string (uri, &protocol, &host, &port, error_msg))
+  if (!vinagre_connection_split_string (uri, NULL, &protocol, &host, &port, error_msg))
     return NULL;
 
   if (use_bookmarks)
@@ -630,7 +619,7 @@ vinagre_connection_new_from_file (const gchar *uri, gchar **error_msg, gboolean
 			     &data,
 			     NULL,
 			     NULL,
-			     &error));
+			     &error))
     {
       if (error)
 	{
@@ -661,7 +650,7 @@ the_end:
   g_free (data);
   g_object_unref (file_a);
 
-  if (!conn)
+  if (!conn && !*error_msg)
     *error_msg = g_strdup (_("The file was not recognized by any of the plugins."));
 
   return conn;
